package java.util.concurrent.atomic;

import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.Serializable;

public class DoubleAdder extends Striped64 implements Serializable
{
    private static final long serialVersionUID = 7249069246863182397L;
    
    public DoubleAdder() {
    }
    
    public void add(double n) {
        Striped64.Cell[] cells;
        if ((cells = this.cells) == null) {
            long base = this.base;
            if (this.casBase(base, Double.doubleToRawLongBits(Double.longBitsToDouble(base) + n))) {
                return;
            }
        }
        boolean cas = true;
        int n2;
        Striped64.Cell cell;
        if (cells != null && (n2 = cells.length - 1) >= 0 && (cell = cells[getProbe() & n2]) != null) {
            Striped64.Cell cell2 = cell;
            long value = cell.value;
            if ((cas = cell2.cas(value, Double.doubleToRawLongBits(Double.longBitsToDouble(value) + n))) != 0) {
                return;
            }
        }
        this.doubleAccumulate(n, null, (boolean)cas);
    }
    
    public double sum() {
        Striped64.Cell[] cells = this.cells;
        double longBitsToDouble = Double.longBitsToDouble(this.base);
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    longBitsToDouble += Double.longBitsToDouble(cell.value);
                }
            }
        }
        return longBitsToDouble;
    }
    
    public void reset() {
        Striped64.Cell[] cells = this.cells;
        this.base = 0L;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    cell.value = 0L;
                }
            }
        }
    }
    
    public double sumThenReset() {
        Striped64.Cell[] cells = this.cells;
        double longBitsToDouble = Double.longBitsToDouble(this.base);
        this.base = 0L;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    long value = cell.value;
                    cell.value = 0L;
                    longBitsToDouble += Double.longBitsToDouble(value);
                }
            }
        }
        return longBitsToDouble;
    }
    
    @Override
    public String toString() {
        return Double.toString(this.sum());
    }
    
    @Override
    public double doubleValue() {
        return this.sum();
    }
    
    @Override
    public long longValue() {
        return (long)this.sum();
    }
    
    @Override
    public int intValue() {
        return (int)this.sum();
    }
    
    @Override
    public float floatValue() {
        return (float)this.sum();
    }
    
    private Object writeReplace() {
        return new DoubleAdder.SerializationProxy(this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Proxy required");
    }
    
    private static class SerializationProxy implements Serializable
    {
        private static final long serialVersionUID = 7249069246863182397L;
        private final double value;
        
        SerializationProxy(DoubleAdder doubleAdder) {
            this.value = doubleAdder.sum();
        }
        
        private Object readResolve() {
            DoubleAdder doubleAdder = new DoubleAdder();
            doubleAdder.base = Double.doubleToRawLongBits(this.value);
            return doubleAdder;
        }
    }
}
